#include "cbarlib.h"
#include "cbarprivate.h"

#define KROSS_MAIN_EXPORT KDE_EXPORT
//#include <main/manager.h>
#include <main/scriptguiclient.h>
#include <api/qtobject.h>
#include <api/interpreter.h>
#include <api/dict.h>
#include <main/manager.h>

#include <qpushbutton.h>
#include <qmessagebox.h>
#include <qlayout.h>

#include <kfiledialog.h>

extern "C"
{
    /**
     * Exported an loadable function as entry point to use
     * the module.
     */
    Kross::Api::Object* init_module(Kross::Api::Manager* manager)
    {
        return new ScriptingModule(manager);
    }
}

/******************** ScriptingModule **************************/
ScriptingModule::ScriptingModule(
    Kross::Api::Manager* manager
    )
    :Kross::Api::Module(
        "ScriptingModule"
        ), 
        m_manager(manager)
{
    Kross::Api::Object::Ptr cbarWidgetPtr = ((Kross::Api::Object*)manager)->getChild("CbarWidget");
    
    bool cbarFound = false;
    
    if (cbarWidgetPtr)
    {
        Kross::Api::QtObject* cbarQtObject = dynamic_cast<Kross::Api::QtObject*>(cbarWidgetPtr.data());
        
        if (cbarQtObject != NULL) 
        {
            QObject *qobj = cbarQtObject->getObject();
            //dynamic cast does not work across DLL boundary
//            CbarWidget* cbarWidget = dynamic_cast<CbarWidget*>(qobj);
            
            if (qobj->inherits("CbarWidget"))
            {
                CbarWidget *cbarWidget = static_cast<CbarWidget*>(qobj);
                addChild(new CbarBinding(cbarWidget));
                cbarFound = true;
            }
        }
    }
    
    if (!cbarFound)
    {
        throw Kross::Api::Exception::Ptr(
            new Kross::Api::Exception("There was no correct CbarWidget published.")
            );
    }
}

ScriptingModule::~ScriptingModule()
{
}

/************************* CbarBinding *********************/

CbarBinding::CbarBinding(
    CbarWidget *widget
    ):
    Kross::Api::Class<CbarBinding>("CbarBinding", 0 ), 
    m_cbarWidget(widget)
{
    addFunction("addCbarWidget", &CbarBinding::addCbarWidget);
    addFunction("registerThemeManager", &CbarBinding::registerThemeManager);
    addFunction("registerScriptLoader", &CbarBinding::registerScriptLoader);
    addFunction("getCbarName", &CbarBinding::getCbarName);
    addFunction("executeScript", &CbarBinding::executeScript);
}

CbarBinding::~CbarBinding() 
{
}

const QString CbarBinding::getClassName() const 
{
    return "CbarBinding";
}

Kross::Api::Object::Ptr CbarBinding::addCbarWidget(Kross::Api::List::Ptr args)
{
    kdDebug() << "Entering CbarBinding::addCbarWidget()\n";
    
    uint ptrAsInt = Kross::Api::Variant::toULLONG(args->item(0));
    QWidget *widget = reinterpret_cast<QWidget*>(ptrAsInt);
    
    Kross::Api::Dict *dict = Kross::Api::Object::fromObject<Kross::Api::Dict>(
        args->item(1)
        );
    kdDebug() << "Dict=" << dict->toString() << "\n";
    
    QMap<QString, Object::Ptr> objectDict = dict->getValue();
    //Well, pyQt apparently does not support QMap and QList in signals :/
    QStringList params;
    
    QMap<QString, Object::Ptr>::iterator it;
    for (it = objectDict.begin(); it != objectDict.end(); ++it)
    {
        QString value =  Kross::Api::Variant::toString(it.data());
/*        kdDebug() << "Key=" << it.key() << ", value=" << value << "\n";*/
        params.push_back(it.key() + "=" + value);
    }
/*    kdDebug() << "Emitting params, params=" << params << "\n";*/
    
    QObject *connectionObject = m_cbarWidget->addCbarWidget(
        widget,
        params
        );
    kdDebug() << "CbarBinding::addCbarWidget(), connectionObject=" << connectionObject << "\n";
    
    return Kross::Api::Object::Ptr(new Kross::Api::Variant((unsigned long long)connectionObject));
}

Kross::Api::Object::Ptr CbarBinding::executeScript(Kross::Api::List::Ptr args)
{
    kdDebug() << "Entering CbarBinding::executeScript()\n";
    
    QString scriptPath = Kross::Api::Variant::toString(args->item(0));
    
    bool result = m_cbarWidget->executeScript(
        scriptPath
        );
    
    return Kross::Api::Object::Ptr(new Kross::Api::Variant(result));
}

Kross::Api::Object::Ptr CbarBinding::getCbarName(Kross::Api::List::Ptr /*args*/)
{
    kdDebug() << "Entering CbarBinding::getCbarName()\n";
    return Kross::Api::Object::Ptr(new Kross::Api::Variant(m_cbarWidget->getCbarName()));
}

Kross::Api::Object::Ptr CbarBinding::registerThemeManager(Kross::Api::List::Ptr args)
{
    kdDebug() << "Entering CbarBinding::registerThemeManager()\n";
    
    uint ptrAsInt = Kross::Api::Variant::toULLONG(args->item(0));
    QWidget *widget = reinterpret_cast<QWidget*>(ptrAsInt);
    QObject *connectionObject = m_cbarWidget->registerThemeManager(widget);
    
    return Kross::Api::Object::Ptr(new Kross::Api::Variant((unsigned long long)connectionObject));
}

Kross::Api::Object::Ptr CbarBinding::registerScriptLoader(Kross::Api::List::Ptr args)
{
    kdDebug() << "Entering CbarBinding::registerScriptLoader()\n";
    
    uint ptrAsInt = Kross::Api::Variant::toULLONG(args->item(0));
    QObject *scriptLoaderObject = reinterpret_cast<QObject*>(ptrAsInt);
    QObject *connectionObject = m_cbarWidget->registerScriptLoader(scriptLoaderObject);
    
    return Kross::Api::Object::Ptr(new Kross::Api::Variant((unsigned long long)connectionObject));
}
/************************* CbarWidget *********************/

QObject *CbarWidget::addCbarWidget(
    QWidget *newWidget,
    QStringList const & params
    )
{
/*    kdDebug() << "Emitting params, params=" << params << "\n";*/
    emit signalAddCbarWidget(newWidget, params);
    return this;
}

QObject *CbarWidget::registerThemeManager(QWidget *themeManager)
{
    themeManager->reparent(this, 0/*WFlags*/, QPoint(0, 0), true /*showIt*/);
    this->layout()->add(themeManager);
    connect(
        themeManager,
        SIGNAL(destroyed()),
        this->d.data(),
        SLOT(themeManagerDestroyed())
        );
    return this;
}

QObject *CbarWidget::registerScriptLoader(QObject *scriptLoaderObject)
{
    this->d->scriptLoader = scriptLoaderObject;
    return this;
}

QString CbarWidget::getCbarName()
{
    return this->d->cbarName;
}

bool CbarWidget::executeScript(
    QString const & scriptPath
    )
{
    return this->d->executeScript(scriptPath);
}


CbarWidget::CbarWidget(
    QString const & cbarName,
    QWidget * parent, 
    const char * name, 
    WFlags flags
    ):
    QWidget(
        parent,
        name,
        flags
        ),
    d(
        new CbarWidgetPrivate(
            this,
            cbarName
            )
        )
{
    this->setPaletteBackgroundColor(Qt::darkBlue);
    this->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding));
    new QVBoxLayout(this);

    Kross::Api::Manager::scriptManager()->addQObject(dynamic_cast<QObject*>(this), "CbarWidget");
        
    this->d->loadScriptLoader();
}

void CbarWidget::openScriptManager()
{
    if (this->d->scriptLoader != NULL)
    {
        emit signalOpenScriptManager();
        return;
    }
    //fallback in case script manager is not registered
    this->openDefaultScriptManager();
}

void CbarWidget::openDefaultScriptManager()
{
    QStringList mimetypes;
    QMap<QString, Kross::Api::InterpreterInfo*> infos = Kross::Api::Manager::scriptManager()->getInterpreterInfos();
    
    for(QMap<QString, Kross::Api::InterpreterInfo*>::Iterator it = infos.begin(); it != infos.end(); ++it)
    {
        mimetypes.append( (*it)->getMimeTypes().join(" ") );
    }
    
    KFileDialog* filedialog = new KFileDialog(
        QString(),
        mimetypes.join(" "),
        NULL,
        "filedialog",
        FALSE //modal
        );
    
    filedialog->setCaption("Select script to run");
    if (!filedialog->exec())
    {
        delete filedialog;
        return;
    }
    
    QString filename = filedialog->selectedFile();
    delete filedialog;

    this->d->executeScript(filename);
}

void CbarWidget::itemsSelected(const KFileItemList & items)
{
    emit signalItemsSelected(items);
}

void CbarWidget::urlOpened(const QString & url)
{
    emit signalUrlOpened(url);
}

/************************* CbarFactory *********************/

CbarWidget::Ptr CbarFactory::createCbarWidget(
    QString const & cbarName,
    QWidget *parent
    )
{
    return CbarWidget::Ptr(
        new CbarWidget(
            cbarName, 
            parent
            )
        );
}

#include "cbarlib.moc"
