#include "cbarprivate.h"

#include <qpushbutton.h>
#include <qmessagebox.h>
#include <kstandarddirs.h>
#include <kconfig.h>

/************************* CbarWidgetPrivate *********************/

CbarWidgetPrivate::CbarWidgetPrivate(
    CbarWidget *_parent,
    QString const & _cbarName
    ):
    parent(_parent),
    cbarName(_cbarName),
    scriptLoader(NULL)
{
}

CbarWidgetPrivate::~CbarWidgetPrivate()
{
}

bool CbarWidgetPrivate::executeScript(
    QString const & scriptFileName
    )
{
    Kross::Api::ScriptAction::Ptr action = Kross::Api::ScriptAction::Ptr(
        new Kross::Api::ScriptAction(scriptFileName)
        );
        
    Script::Ptr scriptPtr = Script::Ptr(
        new Script(action)
        );
        
    this->scripts.insert(scriptFileName, scriptPtr, true);
    
    connect(
        action.data(), 
        SIGNAL(failed(const QString&, const QString&)),
        this, 
        SLOT(scriptExecutionFailed(const QString&, const QString&))
        );
    connect(
        action.data(), 
        SIGNAL(success()),
        this, 
        SLOT(scriptExecutionSuccess())
        );
/*    connect(
        action.data(), 
        SIGNAL(activated(const Kross::Api::ScriptAction*)), 
        SIGNAL(executionStarted(const Kross::Api::ScriptAction*))
        );*/
    
    action->activate();
    bool ok = action->hadException();
//    action->finalize(); // execution is done.
    return ok;
}

void CbarWidgetPrivate::themeManagerDestroyed()
{
    kdDebug() << "CbarWidgetPrivate::themeManagerDestroyed()\n";
    QMessageBox::warning(NULL, "themeManagerDestroyed", "Theme manager destroyed");
}

void CbarWidgetPrivate::scriptExecutionFailed(const QString&, const QString&)
{
    kdDebug() << "CbarWidgetPrivate::executionFailed()\n";
    QMessageBox::warning(NULL, "Script failed", "Execution of script failed");
}

void CbarWidgetPrivate::scriptExecutionSuccess()
{
    kdDebug() << "CbarWidgetPrivate::successfullyExecuted()\n";
/*    QMessageBox::information(NULL, "Script executed", "Script executed sucessfully");*/
}

void CbarWidgetPrivate::executionStarted(const Kross::Api::ScriptAction*)
{
    kdDebug() << "CbarWidgetPrivate::executionStarted()\n";
    QMessageBox::information(NULL, "Script started", "Script execution started");
}

CbarWidgetPrivate::Script::Script(
    Kross::Api::ScriptAction::Ptr const & _scriptAction
    ):
    scriptAction(_scriptAction)
{
}

KConfig *CbarWidgetPrivate::getConfig()
{
    return new KConfig(this->cbarName + "/cbarrc");
}

void CbarWidgetPrivate::loadScriptLoader()
{
    KConfig *config = this->getConfig();
    
    config->setGroup("ScriptLoader");
    QString scriptLoaderPath = config->readEntry("scriptPath");
    kdDebug() << "scriptLoaderPath=" << scriptLoaderPath << "\n";
    
    if (scriptLoaderPath.isEmpty() == true)
    {
        kdDebug() << "Cannot find script loader config file, trying default\n";
        scriptLoaderPath = ::locate("data", "cbar/scripts/defaultScriptLoader.py");
    }
    
    kdDebug() << "Executing script loader script, scriptLoaderPath=" << scriptLoaderPath << "\n";
    this->executeScript(scriptLoaderPath);
    
    delete config;
}



#include "cbarprivate.moc"
