#ifndef __CBAR_PRIVATE_H__
#define __CBAR_PRIVATE_H__

#include <api/module.h>
#include <qobject.h>
#include <ksharedptr.h>
#include <main/scriptaction.h>
#include <qmap.h>

class QPushButton;
class CbarWidget;

namespace Kross {
     namespace Api {
         class ScriptAction;
         class Manager;
     }
}

class ScriptingModule : public Kross::Api::Module
{
    public:
        ScriptingModule(Kross::Api::Manager* manager);
        virtual ~ScriptingModule();
    private:
        Kross::Api::Manager* m_manager;
};

class CbarBinding : public Kross::Api::Class<CbarBinding>
{
    public:
        explicit CbarBinding(CbarWidget *widget);
        virtual ~CbarBinding();
        virtual const QString getClassName() const;
    private:
        Kross::Api::Object::Ptr addCbarWidget(Kross::Api::List::Ptr args);
        Kross::Api::Object::Ptr registerThemeManager(Kross::Api::List::Ptr args);
        Kross::Api::Object::Ptr registerScriptLoader(Kross::Api::List::Ptr args);
        Kross::Api::Object::Ptr getCbarName(Kross::Api::List::Ptr args);
        Kross::Api::Object::Ptr executeScript(Kross::Api::List::Ptr args);
    private:
        CbarWidget* m_cbarWidget;
};

class CbarWidgetPrivate:
    public QObject,
    public KShared
{
    Q_OBJECT;
public:
    CbarWidgetPrivate(
        CbarWidget *parent,
        QString const & cbarName
        );
    virtual ~CbarWidgetPrivate();
    
public slots:
    bool executeScript(
        QString const & scriptFileName
        );
    void scriptExecutionFailed(const QString&, const QString&);
    void scriptExecutionSuccess();
    void executionStarted(const Kross::Api::ScriptAction*);
    void themeManagerDestroyed();
protected:
    CbarWidget *parent;
    QString cbarName;
    QObject *scriptLoader;
    
    class Script:
        public KShared
    {
    public:
        typedef KSharedPtr<Script> Ptr;
        
        Script(
            Kross::Api::ScriptAction::Ptr const & scriptAction
            );
        
        Kross::Api::ScriptAction::Ptr scriptAction;
    };
    
    typedef QMap<QString, Script::Ptr> ScriptsContainer;
    ScriptsContainer scripts;
    
    /**
        Loads config for this instance of Cbar.
    */
    KConfig *getConfig();

    /**
        Loads script loader script.
    */
    void loadScriptLoader();

    
    friend class CbarWidget;
};

#endif //#ifndef __CBAR_PRIVATE_H__
