from qt import *
from kio import *

class DefaultThemeManager(QWidget):
    import qt
    class ActionGroup(qt.QWidget):
        import qt
        class ShowHideEventFilter(qt.QObject):
            def __init__( self, actionGroup, *args ):
                import qt
                apply(qt.QObject.__init__, (self,) + args)
                self.actionGroup = actionGroup
            def eventFilter(self, qobject, event):
                #pass to parent
                return self.actionGroup.showHideEventFilter(qobject, event)
        def __init__(self, actionGroupName, *args):
            import qt
            import kdeui
            apply(qt.QWidget.__init__, (self,) + args )
            self.actionGroupName = actionGroupName
            layout = qt.QGridLayout(
                self,
                1, #numRows
                1 #numCols 
                )
            self.setPaletteBackgroundColor(qt.QColor("lightBlue"))
            self.setSizePolicy(qt.QSizePolicy(qt.QSizePolicy.Expanding, qt.QSizePolicy.Expanding))
            
            #Contents widget
            self.contentsWidget = qt.QWidget(self)
            self.contentsWidget.setSizePolicy(qt.QSizePolicy(qt.QSizePolicy.Expanding, qt.QSizePolicy.Expanding))
            self.contentsLayout = qt.QGridLayout(
                self.contentsWidget,
                1, #numRows
                1 #numCols 
                )
            self.layout().addWidget(self.contentsWidget, 1, 0, qt.Qt.AlignCenter | qt.Qt.AlignTop)
            
            #Header
            self.headerLayout = qt.QGridLayout(None, 1, 3) #row, col
            
            self.header = qt.QLabel(actionGroupName, self, "headerTitle")
            self.header.setSizePolicy(qt.QSizePolicy(qt.QSizePolicy.Expanding, qt.QSizePolicy.Fixed))
            self.header.setPaletteBackgroundColor(qt.QColor(0, 140, 255))
            self.header.setIndent(5)
            
            self.hideButton = kdeui.KURLLabel(self)
            self.hideButton.setUseTips()
            self.hideButton.setFloat(False)
            self.hideButton.setUnderline(False)
            self.actionGroupOpened = False
            self.toggleActionGroupOpened(True)
            qt.QObject.connect(
                self.hideButton, 
                qt.SIGNAL("leftClickedURL(const QString&)"),
                self.clickedHideButton
                )
            self.headerLayout.addWidget(self.hideButton, 0, 2) #int row, int col
            
            #self.header.setFrameStyle( qt.QFrame.Panel | qt.QFrame.Sunken )
            self.headerLayout.addWidget(self.header, 0, 0) #int row, int col
            self.layout().addLayout(self.headerLayout, 0, 0) #int row, int col
            self.header.show()
            
            self.insertWidgetIndex = 0
            self.show()
            self.eventFilter = self.ShowHideEventFilter(self)
            self.childWidgets = []
    
        def showHideEventFilter(self, qobject, event):
            import qt
            if event.type() == qt.QEvent.Hide:
                self.checkAutoHide()
            elif event.type() == qt.QEvent.Show:
                self.checkAutoHide()
            if event.type() == qt.QEvent.HideToParent:
                self.checkAutoHide()
            elif event.type() == qt.QEvent.ShowToParent:
                self.checkAutoHide()
                
            return False #Do not eat event
        def checkAutoHide(self):
            hide = True
            for widget in self.childWidgets:
                if widget.isHidden() == False:
                    hide = False
                    break
            if hide == True and self.isHidden() == False:
                self.hide()
            if hide == False and self.isHidden() == True:
                self.show()
                
        def clickedHideButton(self):
            self.toggleActionGroupOpened(not self.actionGroupOpened)
            
        def toggleActionGroupOpened(self, opened):
            import kdecore
            if self.actionGroupOpened == opened:
                #nothing to do, state is already correct
                return
            if self.actionGroupOpened == False:
                self.hideButton.setPixmap(
                    kdecore.KGlobal.iconLoader().loadIcon(
                        "2uparrow", 
                        kdecore.KIcon.Desktop, 
                        kdecore.KIcon.SizeSmall #size
                        )
                    )
                self.hideButton.setTipText("Hide")
                self.actionGroupOpened = True
                self.contentsWidget.show()
            else:
                self.hideButton.setPixmap(
                    kdecore.KGlobal.iconLoader().loadIcon(
                        "2downarrow", 
                        kdecore.KIcon.Desktop, 
                        kdecore.KIcon.SizeSmall #size
                        )
                    )
                self.hideButton.setTipText("Show")
                self.actionGroupOpened = False
                self.contentsWidget.hide()
                
        def addWidget(self, widget):
            import qt
            widget.reparent(self.contentsWidget, 0, qt.QPoint(0, 0), 1)
            self.contentsLayout.addWidget(widget, self.insertWidgetIndex, 0, qt.Qt.AlignCenter | qt.Qt.AlignTop) #int row, int col, int alignment
            self.insertWidgetIndex = self.insertWidgetIndex + 1
            widget.installEventFilter(self.eventFilter)
            self.childWidgets.append(widget)

    def __init__( self, *args ):
        import qt
        apply(qt.QWidget.__init__, (self,) + args)
#        self.setBackgroundMode(qt.PaletteBackground)
        self.mainlayout = qt.QGridLayout(
            self,
            4, #numRows
            3, #numCols 
            0, #margin
            5 #space
            )
        self.mainlayout.setColStretch(0, 0) #col, stretch
        self.mainlayout.setColStretch(1, 1) #col, stretch
        self.mainlayout.setColStretch(2, 0) #col, stretch
        self.mainlayout.setColSpacing(0, 10) #col, minSize
        self.mainlayout.setColSpacing(2, 10) #col, minSize
        self.setPaletteBackgroundColor(qt.QColor("white"))
        self.setSizePolicy(qt.QSizePolicy(qt.QSizePolicy.Expanding, qt.QSizePolicy.Expanding))
        
        self.label = qt.QLabel("Cbar", self, "label")
        self.layout().addWidget(self.label, 0, 1, qt.Qt.AlignCenter | qt.Qt.AlignTop) #int row, int col, int alignment
        self.label.show()
        
        self.button = qt.QPushButton("Configure scripts", self, "button")
        self.layout().addWidget(self.button, 1, 1, qt.Qt.AlignCenter | qt.Qt.AlignTop)#int row, int col, int alignment
        self.button.show()
        self.actionGroupsLayout = qt.QVBoxLayout()
        self.mainlayout.addLayout(
            self.actionGroupsLayout,
            2, #row
            1 #col
            )
            
        self.mainlayout.addItem(
            qt.QSpacerItem(
                0, #width
                0, #height
                qt.QSizePolicy.Expanding, #hpolicy
                qt.QSizePolicy.Expanding #vpolicy
                ),
            3, #row
            1 #col
            )
        self.mainlayout.setRowStretch(3, 10) #row, stretch
        self.actionGroups = {}
        
    def register(self):
        import qt
        import cbar
        cbarInterface = cbar.Cbar()
        connectionObject = cbarInterface.registerThemeManager(self)
        qt.QObject.connect(
            connectionObject,
            qt.SIGNAL("signalAddCbarWidget(QWidget *, QStringList)"),
            self.addCbarWidget
            )
        qt.QObject.connect(
            self.button, 
            qt.SIGNAL("clicked()"), 
            connectionObject, 
            qt.SLOT("openScriptManager()")
            )
    def convertStringParams(self, paramsAsStrings):
        values = {}
        for s in paramsAsStrings:
            key = str(s.section("=", 0, 0))
            value = str(s.section("=", 1))
            values[key] = value
        return values
            
    def addCbarWidget(self, widget, paramsAsStrings):
        import qt
        params = self.convertStringParams(paramsAsStrings)
        
        if params.has_key("type"):
            paramType = params["type"]
        else:
            paramType = "Other"
            
        actionGroupName = paramType #NOTE: mapping from param type to group names will be added here, do not rely on group appearing with this name!
        
        if self.actionGroups.has_key(actionGroupName):
            actionGroup = self.actionGroups[actionGroupName]
        else:
            actionGroup = self.ActionGroup(actionGroupName, self)
            self.actionGroupsLayout.addWidget(
                actionGroup
                )
            self.actionGroups[actionGroupName] = actionGroup
        
        actionGroup.addWidget(widget)
        #qt.QMessageBox.information(None, "Widget added", "Widget added, params=" + str(params))
    
#QMessageBox.information(None, "Theme manager script run", "Theme manager script run")

themeManager = DefaultThemeManager()
themeManager.register()
themeManager.show()