import cbar
import qt
from kio import *

class FilePreviewWidget(qt.QWidget):
    def __init__( self, *args ):
        import qt
        import kdeui
        apply(qt.QWidget.__init__, (self,) + args )
        layout = qt.QVBoxLayout(self)
        self.setSizePolicy(qt.QSizePolicy(qt.QSizePolicy.Expanding, qt.QSizePolicy.Expanding))
        
        self.label = qt.QLabel("No preview", self, "label")
        self.label.show()
        self.label.setMinimumSize(qt.QSize(100, 100))
        self.layout().addWidget(self.label, 0, qt.Qt.AlignLeft | qt.Qt.AlignTop)
        
    def itemsSelected(self, items):
        import qt
        import kio
        self.label.clear()
        self.label.setText("No preview")
        
        if len(items) != 1:
            return
        selectedItem = items[0]
        if not selectedItem.mimetype().startsWith("image/"):
            return
        
        self.label.clear()
        self.label.setText("Generating preview...")
        
        previewJob = kio.KIO.filePreview(items, 100)
          
        qt.QObject.connect(
            previewJob, 
            qt.SIGNAL("gotPreview(const KFileItem*, const QPixmap&)"),
            self.slotSetPreview
            )
        qt.QObject.connect(
            previewJob, 
            qt.SIGNAL("failed(const KFileItem *)"), 
            self.slotPreviewFailed
            )
        qt.QObject.connect(
            previewJob, 
            qt.SIGNAL("result(KIO::Job *)"), 
            self.slotJobFinished
            )
    
    def slotSetPreview(self, fileItem, pixmap):
        import qt
        self.label.clear()
        self.label.setPixmap(pixmap)
        self.label.show()
        
    def slotPreviewFailed(self, fileItem):
        import qt
        qt.QMessageBox.information(self, "Preview failed", "Preview failed")
    def slotJobFinished(self, job):
        pass
    def urlOpened(self, url):
        pass
    

widget = FilePreviewWidget()

cbarInterface = cbar.Cbar()
connectionObject = cbarInterface.addCbarWidget(widget, {"type" : "Preview"})
qt.QObject.connect(
    connectionObject,
    qt.SIGNAL("signalItemsSelected(const KFileItemList &)"),
    widget.itemsSelected
    )
qt.QObject.connect(
    connectionObject, 
    qt.SIGNAL("signalUrlOpened(const QString &)"),
    widget.urlOpened
    )
